'use strict';
const http = require('http');
const https = require('https');
const resolveALPN = require('resolve-alpn');
const urlToOptions = require('./utils/url-to-options');
const HTTP2ClientRequest = require('./client-request');

module.exports = async (input, options) => {
	if (typeof input === 'string' || input instanceof URL) {
		input = urlToOptions(new URL(input));
	}

	options = {...input, ...options};
	options.protocol = options.protocol || 'https:';

	if (options.protocol === 'https:') {
		const {alpnProtocol, socket} = await resolveALPN({
			ALPNProtocols: ['h2', 'http/1.1'],
			...options,
			host: options.hostname || options.host,
			path: options.socketPath,
			port: options.port || 443,
			session: options.socketSession,
			resolveSocket: true
		});

		options.createConnection = () => socket;

		if (alpnProtocol === 'h2') {
			return new HTTP2ClientRequest(options);
		}

		options._defaultAgent = https.globalAgent;
	}

	return new http.ClientRequest(options);
};
