'use strict';
/* istanbul ignore file: https://github.com/nodejs/node/blob/master/lib/internal/errors.js */

module.exports = {};

const makeError = (Base, key, getMessage) => {
	module.exports[key] = class NodeError extends Base {
		constructor(...args) {
			super(typeof getMessage === 'string' ? getMessage : getMessage(args));
		}

		get name() {
			return `${super.name} [${key}]`;
		}

		set name(value) {
			Object.defineProperty(this, 'name', {
				configurable: true,
				enumerable: true,
				value,
				writable: true
			});
		}

		get code() {
			return key;
		}

		set code(value) {
			Object.defineProperty(this, 'code', {
				configurable: true,
				enumerable: true,
				value,
				writable: true
			});
		}
	};
};

makeError(Error, 'ERR_HTTP_HEADERS_SENT', args => {
	return `Cannot ${args[0]} headers after they are sent to the client`;
});

makeError(TypeError, 'ERR_INVALID_HTTP_TOKEN', args => {
	return `${args[0]} must be a valid HTTP token ["${args[1]}"]`;
});

makeError(TypeError, 'ERR_HTTP_INVALID_HEADER_VALUE', args => {
	return `Invalid value "${args[0]}" for header "${args[1]}"`;
});

makeError(TypeError, 'ERR_INVALID_ARG_TYPE', args => {
	const type = args[0].includes('.') ? 'property' : 'argument';
	return `The "${args[0]}" ${type} must be of type ${args[1]}. Received ${typeof args[2]}`;
});

makeError(TypeError, 'ERR_INVALID_PROTOCOL', args => {
	return `Protocol "${args[0]}" not supported. Expected "${args[1]}"`;
});
